%% test convergence of POD basis and estimator Atilde, Btilde
%{
1. Generate multi-traj data; IC sampled
2. Compute POD (test its convergence) and coefficient a(t) for the DE
3. Test convergence of estimator Atilde, Btilde
%}

% compare the convergence of POD and estimators

% lsp_solver = 'standard'; % Least Square Problem solver, 'standard' for standard truncated SVD
saveON     = 1;    % postprocess data
num_test   = 1000;  % number of trajectories (with different ICs); 


%% 1. generate Data: num_test trajectories 
% initialize the FEM solver with pre-determined ICs
fem_settings = burgers1d_fem_mesh_mtraj_v2(opt,domain,fem_settings, filenames);
ic_modes     = 50; % modes of sine functions that the IC contains
u0_mtraj     = get_IC(filenames,fem_settings,ic_modes,num_test,saveON); % get multiple trajectories's ICs
File_snap    =  filenames.File_snap;
File_snapshots_test = [File_snap,'Mtraj',num2str(num_test),'.mat'];
if exist(File_snapshots_test,'file')  % generate data if do not exist
    fprintf('Using previous data for trajectory 1--%i \n',num_test);
    load(File_snapshots_test);  
else
    save_1Traj = 0;  
    Nx           = length(u0_mtraj(:,1)); tN = fem_settings.tN; 
    Kmat_Mtraj   = zeros(Nx,Nx,num_test);
    u_Mtraj      = zeros(Nx,tN,num_test);  % if Nx*tN is too large, we do not save it, save ICs instead
    tic
    ticBytes(gcp); % tocBytes Read how many bytes have been transferred since calling ticBytes
    parfor k =1:num_test
        u0 = u0_mtraj(:,k);
        [k_mat,u_save,~,~,~] = burgers1d_fem_1traj_v2(opt,domain,fem_settings,u0,filenames,save_1Traj,k); 
        Kmat_Mtraj(:,:,k) = k_mat; 
        u_Mtraj(:,:,k)    = u_save; 
    end
    save(File_snapshots_test,'fem_settings','u_Mtraj','Kmat_Mtraj');  
    tocBytes(gcp);     toc
end

dt        = fem_settings.domain.t; % time step size
viscosity = opt.parm_e; % = 2e-3;
x_mesh    = fem_mesh.x;  dx = x_mesh(2)-x_mesh(1); 

%% All the rest uses the solution/data in phase space u(t,x). FEM no longer needed.
%% 2. Test convergence of POD: 
File_pod_mtraj   = [filenames.File_pod_mtraj,num2str(num_test)];
[pod_all,diag_S,pod_conv,sig_conv,k_seq,pod_modes]= pod_conv_mtraj_v2(File_snapshots_test,File_pod_mtraj,saveON);
pod_conv_plot_v2(pod_conv,sig_conv,k_seq,pod_modes); % a simple plot to see the convergence, combo plot below. 



%% 3. Estiamte ROM: everything is based on POD_Basis, 
pod_all1 = pod_all/sqrt(dx);     % the pod mode in mesh should be scaled by /sqrt(dx) to get them to physical space 
r = 10; 
load(File_snapshots_test,'u_Mtraj'); 
a_Mtraj        = get_at_data(u_Mtraj,pod_all1,r)*dx;               % get a(t) trajectory data

num_test1 = 1000;  % or = num_test1; = 1500
timegap = 5; a_Mtraj_1 = a_Mtraj(:,1:timegap:end,1:num_test1);
delta   = timegap*dt;

anastz_type    = 'Afull_Bsym';  % select from  Afull_Bsym , rk4_AB, Adiag_Bsym, NAR
[romInfo,A,B]  = get_ROM_info(pod_all1,delta,viscosity,r,x_mesh,anastz_type);    % eig(A);   %  we need all eigenvalues to be negative, and to decay fast
romInfo.timegap = timegap;
File_estimator = [filenames.Folder_save,sprintf('Estimator_M%i_r%i_timegap%i_',num_test1,r,timegap),anastz_type,'0322.mat'];
romInfo        = est_conv_mtraj_v2(a_Mtraj_1,delta, romInfo,File_estimator);        % get the estimators and save them
plotEst_conv_combo; 

figpath = [filenames.Folder_save,'/Figures/']; 
% output A_est and B_est
printArray2tex(romInfo.A_est,romInfo.B_est,figpath); 


%% 4. Prediction performce of ROM
% % prediciton using new data if saved previously 
% Mpred = 100; mInd = 1001:1000+Mpred; 
% run_rom_performance(a_Mtraj(:,1:timegap:end,mInd),romInfo,filenames,u_Mtraj(:,1:timegap:end,mInd),pod_all1(:,1:r)); 

% prediciton using new data with a larger time interval
M_pred_data    = 200; 
tN_pred        = 800;  
[a_Mtraj_2,u_r_err20,u_Mtraj2] = get_predictData(fem_settings, filenames,pod_all1,tN_pred,M_pred_data);    % get 200 new trajectories, each have 20 modes

File_predErr  = [filenames.Folder_save,sprintf('spaceTime_predM%i_',M_pred_data),anastz_type,'SigmaOFF_all.mat'];
Mpred = 100; tind = 1:timegap:(800+1);
run_rom_performance(a_Mtraj_2(1:r,tind,1:Mpred),romInfo,filenames,u_Mtraj2(:,tind,1:Mpred),pod_all1(:,1:r),x_mesh);
 

%% 5. Space-time reduction 
a_Mtraj_st        = get_at_data(u_Mtraj,pod_all1,16)*dx;        % the test r should be <20
space_time_reduction(a_Mtraj_st,dt,anastz_type,pod_all1,viscosity,x_mesh,filenames,fem_settings); 

% [a_Mtraj] = get_predictData(fem_settings, filenames,pod_all);
%  
% gapSeq   = 1:1:15; 
% [File_est_tseq,romInfo_seq,gapSeq] = getEst_timegapSeq(gapSeq,a_Mtraj,dt,anastz_type,pod_all1,viscosity,r,x_mesh,filenames);
% filenames.File_est_tseq = File_est_tseq; 
    


