function [u, f, ux, B, N]= burgers1d_fem_setting(opt,domain)
%--------------------------------------------------------------------------
% Generate exact solution
%--------------------------------------------------------------------------
syms t x
%--- determine exact solution
switch opt.es
    case 0 % exact solution is unknown
        u_sym  = [];
end
%--- If u is known,
if isempty(u_sym)~=1 % isempty returns whether or not u_sym is empty
    u_t   = diff(u_sym, 't', 1);
    u_x   = diff(u_sym, 'x', 1);
    u_xx  = diff(u_sym, 'x', 2);
else 
    u_x   = []; 
end
% convert symbolic expressions to Matlab functions for u
if isempty(u_sym)~=1
    [u.func, u.var] = fem_sym_2_func(u_sym);
    [ux.func, ux.var] = fem_sym_2_func(u_x);
else
    u.func = [];  u.var = [];
    ux.func = []; ux.var = [];
end
%--- type of RHS
switch opt.rhs
    case 0 %--- RHS is 0
        f_str = '0'; 
        f_sym = 0*x;
        % nonlinear function and its derivative 
        N     = []; % empty for Burgers, specify nonlinear func. here, otherwise
        %         f.func = @(x)0; f.var ={'x'};
    case 1
        f_str = 'u_t -e*u_xx +u*u_x';
        f_sym = u_t - opt.parm_e*u_xx + u_sym*u_x;  
        % nonlinear function and its derivative 
        N     = []; % empty for Burgers, specify them here, otherwise
end
%--- type of boundary condition
switch opt.bc
    case 0
        B.criterion = {'Full boundary'};
        B.type      = 'D';
        B.value     = {0};
    case 1
        Xleft   = domain.interval(1,1);
        Xright  = domain.interval(1,2);
        B.criterion = {['abs(x-',num2str(Xleft),')<=1e-5'],...
            ['abs(x-',num2str(Xright),')<=1e-5']};
        B.type      = ['N','N'];
        B.value     = {'@(t)50000*t^3*exp(-15*t)', 0};
end
%--- type of initial condition
switch opt.ic
    case 0
        fprintf(1, 'IC = initial guess plus correct Dirichlet BC\n');
    case 1
        fprintf(1, 'IC = a vector loaded from a data \n');
        fprintf(1, 'Check the name of the data in FEM_IC.m \n');
        IC_filename = 'Burgers1D_IC';
    case 2
        fprintf(1, 'IC = initial conditions \n');
        [u.ifunc, u.ivar] = fem_sym_2_func('0*x');
    case 3 % initial condition is given by the function is not written as a function handle
        fprintf(1, 'IC = initial conditions evaluated by Burgers1D_IC.m \n');
end
%--- convert symbolic expressions to Matlab functions for f
if isempty(f_sym)~=1
    [f.func, f.var]   = fem_sym_2_func(f_sym);
end

if isempty(u_sym) == 1
    u.str = 'unknown';
else
    u.str = char(u_sym);
end
f.str = f_str; 
f.sym = f_sym;   % function handle may not be able to load, thus save symbolic expressions
u.sym = u_sym;
ux.sym= u_x;

fprintf(1, ['exact solution is ', u.str, '\n']);
fprintf(1, ['equation is Lu = f  with  ', f.str, '\n']);
pause(1)



