function [M, S, P] = fem1d_assemble_bilinear....
    (x,e_conn,n_gauss,filename_shape,filename_smm)
%=====================================
% Zhu Wang 10/31/2011
% To compute Mass and Stiffness matrices
%
% Inputs
%       x - nodal coordinates
%       e_conn - connectivity matrix
%       n_gauss -  # of Gauss points
% Outputs
%       M(sparse) - mass matrix \int h(x) h(x) dx
%       S(sparse) - stiffness   \int h_x  h_x  dx
%       P(sparse) -             \int h_x  h  dx
%=====================================
[N         , dim    ] = size(x);
[n_elements, nel_dof] = size(e_conn);

%--- initialization
load(filename_shape,'W_w_g','W_phi','W_p_x')

[r, w]   = fem1d_gauss_gen(n_gauss); % generate Gauss points

MA.ii = []; MA.jj = []; MA.xx = []; MA.counter = [];

one      = ones(size(w));
II       = zeros( n_elements*nel_dof^2,1 );
JJ       = zeros( n_elements*nel_dof^2,1 );
XX       = zeros( n_elements*nel_dof^2,1 );
YY       = zeros( n_elements*nel_dof^2,1 );
ZZ       = zeros( n_elements*nel_dof^2,1 );
entry_counter = 0;

for n_el   = 1: n_elements
    nodes_local = e_conn(n_el,:);
    
    w_g     = W_w_g(:,n_el);
    phi     = W_phi(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
    phi_x   = W_p_x(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
    %_________________________________________________________
    M_loc = fem1d_bilinear( one, phi,   phi,   w_g );
    S_loc = fem1d_bilinear( one, phi_x, phi_x, w_g );
    P_loc = fem1d_bilinear( one, phi_x, phi,   w_g );
    
    for i=1:nel_dof
        for j=1:nel_dof
            entry_counter = entry_counter + 1;
            II(entry_counter) = nodes_local(i);
            JJ(entry_counter) = nodes_local(j);
            XX(entry_counter) = M_loc(i,j);
            YY(entry_counter) = S_loc(i,j);
            ZZ(entry_counter) = P_loc(i,j);
        end
    end
end

M = sparse( II(1:entry_counter), JJ(1:entry_counter),...
    XX(1:entry_counter),...
    N, N );
S = sparse( II(1:entry_counter), JJ(1:entry_counter),...
    YY(1:entry_counter),...
    N, N );
P = sparse( II(1:entry_counter), JJ(1:entry_counter),...
    ZZ(1:entry_counter),...
    N, N );

if nargin>4
    % prepartion for FEIC (Sparse Mass Multiplication)
    MA.ii = II; MA.jj = JJ;
    MA.xx = XX; MA.counter = entry_counter;
    
    save(filename_smm, 'MA','N');
end



