function [Nh, Nhx] = fem1d_assemble_bilinear_nonlker...
    (x,e_conn,n_gauss,u,Nonl_func,filename_shape)
%=====================================
% Zhu Wang 10/31/2011
% To compute Mass and Stiffness matrices
%
% Inputs
%       x - nodal coordinates
%       e_conn - connectivity matrix
%       n_gauss -  # of Gauss points
% Outputs
%       Nh(sparse)  - \int N(u) h(x) dx
%       Nhx(sparse) - \int N(u) h_x  dx
%=====================================
[N         , dim    ] = size(x);
[n_elements, nel_dof] = size(e_conn);

%--- initialization
load(filename_shape,'W_w_g','W_phi','W_p_x')

[r, w]   = FEM_oned_gauss(n_gauss); % generate Gauss points

one      = ones(size(w));
Nh       = zeros(N,1);
Nhx      = zeros(N,1);

for n_el   = 1: n_elements
    nodes_local = e_conn(n_el,:);
    
    w_g     = W_w_g(:,n_el);
    phi     = W_phi(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
    phi_x   = W_p_x(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
    %----------------
    u_loc   = u(nodes_local, 1);
%     nonl1   = feval(Nonl_func, phi*u_loc); % evaluate nonlinear function at phi*u_loc
    nonl1   = eval_func(Nonl_func, phi*u_loc);
    Nh_loc  = FEM_oned_bilinear( one, nonl1, phi,   w_g );  %(N(u), h)   
    Nhx_loc = FEM_oned_bilinear( one, nonl1, phi_x, w_g );

    Nh(nodes_local)  = Nh(nodes_local)  + Nh_loc;
    Nhx(nodes_local) = Nhx(nodes_local) + Nhx_loc;
end

Nh = sparse( Nh );
Nhx= sparse( Nhx);



