
function [romInfo,A,B] = get_ROM_info(pod_all,dt,viscosity,r,x_mesh,anastz_type)
% get ROM information, such as A, B, ansatz
%{
          dX = AX + X'*B*X + closure terms      dX = X(t+1) - X(t) / dt
Here X is the trajectory of X = (a_i = <u,phi_i>).   
           <u_t,phi_k> = nu * <u_xx,phi_k> - <uu_x,phi_k> + closure terms
>>>>         rmMass*dX  = A*X + X'*B*X + F(X) 
rmMass = Id
A   = - <phi_i',phi_k'>;           r x r         %  - <,>  by integration by parts
B   = 0.5*<phi_i*phi_j,phi_k'>;      r x r x r   % no -    by integration by parts
%}

nx     = length(x_mesh); dx = x_mesh(2)-x_mesh(1);
rmMass = pod_all'*pod_all*dx;% eye(nx);

pod_rm  = pod_all(:,1:r);   
diffphi = (pod_rm(2:end,1:r) - pod_rm(1:end-1,1:r))/dx;   % difference, size nx-1 x r
A =  - viscosity*(diffphi'*diffphi)*dx; 
B = zeros(r,r,r);
for ii =1:r
   B(:,:,ii) = pod_rm(1:end-1,:)'*(pod_rm(1:end-1,:).*diffphi(:,ii)); 
end
B = 0.5* B*dx; 

romInfo.r      =  r; 
romInfo.rmMass = rmMass; 
romInfo.A      = A; 
romInfo.B      = B; 
romInfo.pod_rm = pod_rm; 
romInfo.dt     = dt;


At  = zeros(r,r); 
romInfo.Ind_triag_up = (1:size(At,1)).' <= (1:size(At,2));    % indicator matrix with logical 1 for the upper triangular, so as to select the upper triangular elements

romInfo.anastz_type = anastz_type; % 'rk4+AB';  % 'Afull_Bsym' , 'rk4+AB', Adiag_Bsym
% see their terms in ROM_paraTerms();

% [eigL] = eigLaplace(pod_all,dx);     % test the eigenfunctions of A_full >>> they are highly oscillatory, not the Laplace's eigenfunctions


%% for NAR model: key we need polynomials higher than order 2 to get the interaction between low and high
%{
           dX_k = c_k*RK4(X,dt)_k + A_est*X+ Cmat*TermQ(X) + noise
    where  TermQ = [X,a2r_est]'*B2r*[X,a2r_est]
%}
pod_rm2r  = pod_all(:,1:2*r);   
diffphi = (pod_rm2r(2:end,:) - pod_rm2r(1:end-1,:))/dx;   % difference, size nx-1 x r
A2r =  - viscosity*(diffphi'*diffphi)*dx; 
B2r = zeros(2*r,2*r,2*r);
for ii =1:r
   B2r(:,:,ii) = pod_rm2r(1:end-1,:)'*(pod_rm2r(1:end-1,:).*diffphi(:,ii)); 
end
B2r = 0.5* B2r*dx; 
romInfo.A2r   = A2r; 
romInfo.B2r   = B2r; 
romInfo.ASL   = A2r(r+1:2*r,1:r);       %  ALS = romInfo.ASL'; 
romInfo.BSL   = B2r(1:r,1:r,r+1:2*r);   % B^S_L: large scale part of B for small scales
romInfo.B2rL  = B2r(:,:,1:r);         % B2r the large scale part
end



%% For exploration. Not used in paper. 
function [eigL] = eigLaplace(pod_all,dx)
% test the eigenfuntions of Laplace in the POD basis

nx     = length(pod_all(:,1));
r      = nx-1; 
pod_rm  = pod_all(:,1:r);   
diffphi = (pod_rm(2:end,1:r) - pod_rm(1:end-1,1:r))/dx;   % difference, size nx-1 x r
A = - (diffphi'*diffphi)*dx;

[V, eigLmat] = eig(A);  eigL      = diag(eigLmat);        
[eigL, Ind]=sort(eigL,'descend');   V = V(:,Ind);   
figure; plot(V(:,1:2)); hold on;
plot(pod_all(:,1:2)); 
figure; plot(eigL);
end


