 function run_rom_performance(a_Mtraj,romInfo,filenames,uMtraj,pod_all1,xmesh)
% test ROM performance 
%{
For 1-4: deterministic simulation 
1. Single trajectory: solution field, 
2. Single trajectory: spacial curve at a time, a_i trajecotry, L2 error trajectory
3. L2 error ensemble stats
4. stochastic trajectory: see if true trajectory uncertainty
%}


colors;
[r,tNrom,M_pred]  = size(a_Mtraj(:,:,:));
timegap  = romInfo.timegap;  
dt       = romInfo.dt;  dx = xmesh(2) - xmesh(1); 
nx       = length(uMtraj(:,1,1)); 

figpath = [filenames.Folder_save,'/Figures/'];
tt = dt*(1:1:tNrom)-dt;

% [nx,~,Nseq] = size(A_est_Mseq);
% A_est =  A_est_Mseq(:,:,end); B_est =  B_est_Mseq(:,:,:,end);

% Galerkin truncated ROM
romInfo1   = romInfo; romInfo1.anastz_type= 'Galerkin';
romInfo1.A_est  = 0*romInfo1.A_est; romInfo1.B_est= 0*romInfo1.B_est;
    
errMtraj      = zeros(tNrom,M_pred);    errMtraj_GROM = zeros(tNrom,M_pred);
ur_Mtraj      = zeros(nx,tNrom,M_pred); ur_Mtraj_GROM = zeros(nx,tNrom,M_pred); 
ur_Mtraj_true = ur_Mtraj;  
for m = 1:M_pred
    Xfom           = a_Mtraj(:,:,m); X0 = Xfom(:,1);   
    ur_Mtraj_true(:,:,m) = pod_all1* Xfom;
    % stochastic ROM
    X               = runROM_1traj(X0,dt,tNrom-1,romInfo); %,SigmaT);
    ur_Mtraj(:,:,m) = pod_all1*X; 
    errMtraj(:,m)   = vecnorm(Xfom - X);    
    % Galerkin ROM
    X_galerkin           =  runROM_1traj(X0,dt,tNrom-1,romInfo1); % 0*X_fom; %
    errMtraj_GROM(:,m)   = vecnorm(Xfom - X_galerkin); 
    ur_Mtraj_GROM(:,:,m) = pod_all1*X_galerkin; 
end


errMtraj_GROM(errMtraj_GROM>1) = 1; 
errMtraj_GROM(isnan(errMtraj_GROM)) = 1.2; 
errMtraj(errMtraj>0.4) = 0.4; 
errMtraj(isnan(errMtraj)) = 0.6; 


%% 1. single trajectory: solution field and 
% plot of solution field
fig = figure;  % plot of solution field
space_subplots = 0.02;
ha = tight_subplot(1,4,space_subplots,[0.1,0.08],[0.09,0.06]);
axes(ha(1)); imagesc(tt,xmesh,uMtraj(:,:,M_pred));    caxis([0 0.82]);     yticklabels('auto');ylabel('x');  title('FOM');  xticklabels('auto'); xlabel('Time'); hold on;
axes(ha(2)); imagesc(tt,xmesh,ur_Mtraj_true(:,:,M_pred)); caxis([0 0.82]); yticklabels(''); title('FOM r-modes');xlabel('Time');
axes(ha(3)); imagesc(tt,xmesh,ur_Mtraj(:,:,M_pred)); caxis([0 0.82]);      yticklabels('');title('S-ROM');xlabel('Time');
axes(ha(4)); imagesc(tt,xmesh,ur_Mtraj_GROM(:,:,M_pred)); caxis([0 0.82]); yticklabels('');title('G-ROM');xlabel('Time');
figname = [sprintf('soluField_r%i_timegap%i_tN%i',M_pred,r,timegap,tNrom)]; %,romInfo.anastz_type];
set_positionFontsAll_imagesc; 
% figure; 
% subplot(1,3,1); surf(uMtraj(:,:,M_pred));        title('FOM')
% subplot(1,3,2); surf(ur_Mtraj_true(:,:,M_pred)); title('FOM r-modes')
% subplot(1,3,3); surf(ur_Mtraj(:,:,M_pred));      title('S-ROM'); 
% fig = figure; 
% subplot(1,4,1); imagesc(tt,xmesh,uMtraj(:,:,M_pred));    caxis([0 0.82]);   title('FOM'); xlabel('Time')
% subplot(1,4,2); imagesc(tt,xmesh,ur_Mtraj_true(:,:,M_pred)); caxis([0 0.82]); title('FOM r-modes');xlabel('Time')
% subplot(1,4,3); imagesc(tt,xmesh,ur_Mtraj(:,:,M_pred));     caxis([0 0.82]); title('S-ROM'); xlabel('Time')
% subplot(1,4,4); imagesc(tt,xmesh,ur_Mtraj_GROM(:,:,M_pred)); caxis([0 0.82]); title('GROM'); xlabel('Time')
% set_positionFontsAll_imagesc; 

%% 2. Single trajectory: spacial curve at a time, a_i trajecotry, L2 error trajectory
figure;
% space curve at a time
timeInd = floor(tNrom/4);  
uFOM   = uMtraj(:,timeInd,M_pred);   urFOM  = ur_Mtraj_true(:,timeInd,M_pred); 
uSrom  = ur_Mtraj(:,timeInd,M_pred); uGrom  = ur_Mtraj_GROM(:,timeInd,M_pred);
subplot(1,3,1);  
title(sprintf('(a) Solution at t= %2.1f',tt(timeInd))); hold on;
plot(xmesh, uSrom,'-','linewidth',1); hold on; plot(xmesh,uGrom,'-.','linewidth',1);
plot(xmesh,urFOM,'k:','linewidth',2);   plot(xmesh,uFOM,'--','color',dark_robp(4,:),'linewidth',1);
xlabel('x'); ylabel(sprintf('u(x,%2.1f)',tt(timeInd))) 
legend('S-ROM','G-ROM','FOM r-modes','FOM','location','best'); 

% coefficiet trajectory
inda = 5; 
subplot(1,3,2); 
title(sprintf('(b) Coeffcient of mode %i',inda)); hold on; 
plot(tt,X(inda,:),'-','linewidth',1); hold on;
plot(tt,X_galerkin(inda,:),'-.','linewidth',1); 
plot(tt,Xfom(inda,:),'k:','linewidth',2); xlabel('Time'); ylabel('a_5(t)');  
 legend('S-ROM','G-ROM','FOM r-mode','location','best'); 
 

subplot(1,3,3);   % L2 error trajectory
title('(c) L2 error trajectory'); hold on;  
err_ur_FOM = vecnorm(ur_Mtraj_true(:,:,M_pred)-uMtraj(:,:,M_pred))*sqrt(dx); 
err_urSrom = vecnorm(ur_Mtraj(:,:,M_pred)-uMtraj(:,:,M_pred))*sqrt(dx); 
err_urGrom = vecnorm(ur_Mtraj_GROM(:,:,M_pred)-uMtraj(:,:,M_pred))*sqrt(dx); 
plot(tt, err_urSrom,'-','linewidth',1); 
plot(tt, err_urGrom,'-.','linewidth',1); 
plot(tt, err_ur_FOM,'k:','linewidth',2); 
xlabel('Time'); ylabel('L^2([0,1])');
 legend('S-ROM','G-ROM','FOM r-modes');  

figname = [sprintf('solu1traj_r%i_timegap%i_tN%i',M_pred,r,timegap,tNrom)]; %,romInfo.anastz_type];
set_positionFontsAll; 


%% 3. L2 error ensemble 
% % ensemlbe of all l2 error of a
% figure(101)
% for  m = 1:min(M_pred,100)
%    plot(tt,errMtraj(:,m),'-','color',dark_robp(3,:),'linewidth',1); hold on; 
%    plot(tt,errMtraj_GROM(:,m),'-.','color',dark_robp(2,:),'linewidth',1);
% end
% xlabel('Time'); ylabel('RMSEs')
% legend('S-ROM','Galerkin');    
figure;clf; 
subplot(131);  title('Mean of RMSEs'); hold on % mean error trajectory
errMeanTraj = mean(errMtraj,2);  errStdTraj  = std(errMtraj,0,2); 
errMeanTrajG = mean(errMtraj_GROM,2);  errStdTrajG  = std(errMtraj_GROM,0,2);   
plot(tt,errMeanTraj,'-','color',dark_robp(3,:),'linewidth',1); hold on; 
plot(tt,errMeanTrajG,'-.','color',dark_robp(2,:),'linewidth',1); 
xlabel('Time'); ylabel('Mean of RMSEs'); 
legend('S-ROM','G-ROM');  

% boxplot of the errors  
tind = 1:floor(tNrom/8):tNrom; 
subplot(132); boxplot(errMtraj(tind,:)',(tind-1)*dt); hold on;  xlabel('Time'); ylabel('RMSE'); title('S-ROM');
subplot(133); boxplot(errMtraj_GROM(tind,:)',(tind-1)*dt);   xlabel('Time'); ylabel('RMSE');    title('G-ROM'); 
figname = [sprintf('Pred_err_M%i_r%i_timegap%i_tN%i_',M_pred,r,timegap,tNrom),'box']; % romInfo.anastz_type,
set_positionFontsAll;

% errorbar(errMeanTraj,errStdTraj); hold on; % NOT good, because std is lead to negative error

%% 4. stochastic traj ensemble: with true traj 
% KS statistic of PDF, in time: not a good test 
rom_stocSimu; 





